/*****************************************************************************
*                            dat2vtk exchange program                        *
*                                  for lpis3d version                        *
*                                           2005.9.27                        *
*                              Copyright  Keita Ogawa                        *
*****************************************************************************/
#include "setup.h"

#define stp     (dbp)
#define glp     (dbp+tlp)
#define strdp   (dbp-um)
#define glrdp   (dbp+tlp)
#define header  "# vtk DataFile Version 2.0\n"
#define dformat "DATASET STRUCTURED_POINTS\n"
#define type    "ASCII\n"
#define pdata   ((int)((tlp/dx)+1)*(jmx+1)*(kmx+1))
#define scalar  "SCALARS scalars float\n"
#define lookup  "LOOKUP_TABLE default\n"

#define optErr    -1
#define optSuc     0
#define file_open  1
#define file_close 0
#define _DEBUG

/*****************************************************************************
*                           Define each properties                           *
*****************************************************************************/
static double  Fx[imx+2][jmx+2][kmx+2];    /* Value of field (x)            */
static double  Fy[imx+2][jmx+2][kmx+2];    /* Value of field (y)            */
static double  Fz[imx+2][jmx+2][kmx+2];    /* Value of field (z)            */

static char   fnm[FBUF+1];
static char   pnm[FBUF+1];
static FILE  *fp;
static int    bf;                          /* File buffer number            */

void choice     (char *buf[]);
void getopt     (int  npt,char  *buf[],int argc,char *argv[]);
void extrac     (void);
void out_header (void);
void out_data   (void);
void set_file   (int);

int startup     (int argc,char *argv[]);

double abv      (double,double,double);

/*****************************************************************************
*                               Each programs                                *
*****************************************************************************/
int main(int argc,char *argv[])
{
  if(startup(argc,argv) == optErr) exit(1);
  
  extrac();
  
  set_file(file_open);
  out_header();
  out_data();
  set_file(file_close);
  
  printf("%s ... complated\n",pnm);
  return optSuc;
}

void extrac(void)
{
  int i,j,k;
  
  /********** reading data            **********/
  fp = fopen(fnm,"r");
  if(fp == NULL)
    {
      printf("Can't read a file ...\n");
      exit(1);
    }
  
  for(i=(int)(strdp/dx);i<=(int)(glrdp/dx);i++)
    for(j=0;j<=jmx;j++)
      for(k=0;k<=kmx;k++)
        fscanf(fp,"%lf %lf %lf\n",
	       &Fx[i][j][k],&Fy[i][j][k],&Fz[i][j][k]);
  fclose(fp);
  
#ifdef DEBUG
  printf("file reading is success ... %s\n",fnm);
#endif
}

void set_file(int flgs)
{
  if(flgs == file_open)
    {
      sprintf(pnm,"%c%c%c%c%c%c%c.vtk",
	      fnm[bf-11],fnm[bf-10],fnm[bf-9],
	      fnm[bf- 8],fnm[bf- 7],fnm[bf-6],fnm[bf-5]);
      fp = fopen(pnm,"w");
      if(fp == NULL) exit(1);
      
#ifdef DEBUG
      printf("making is success       ... %s\n",pnm); 
#endif
    }
  
  if(flgs == file_close)
    {
      fclose(fp);
    }
}

void out_header(void)
{
  /********** writing header          **********/
  fprintf(fp,header);

  /********** writing title           **********/
  fprintf(fp,"%s\n",pnm);
  
  /********** writing ASCII or BINARY **********/
  fprintf(fp,type);
  fprintf(fp,"\n");
  
  /********** writing data format     **********/
  fprintf(fp,dformat);
  
  /********** writing Dimensions      **********/
  fprintf(fp,"DIMENSIONS %d %d %d\n",(int)((tlp/dx)+1),jmx+1,kmx+1);
  
  /********** writing Origin          **********/
  fprintf(fp,"ORIGIN %f %f %f\n",stp/twp,0.0,0.0);
  
  /********** writing Spacing         **********/
  fprintf(fp,"SPACING %f %f %f\n",dx/twp,dy/twp,dz/twp);
  fprintf(fp,"\n");
  
  /********** writing Point data      **********/
  fprintf(fp,"POINT_DATA %d\n",pdata);

  /********** writing Scalars         **********/
  fprintf(fp,scalar);
  
  /********** writing Lookup table    **********/
  fprintf(fp,lookup);
  fprintf(fp,"\n");
  
#ifdef DEBUG
  printf("---------------------------------------------------------\n");
  printf("header                  ... ");
  printf(header);
  printf("file type               ... ");
  printf(type);
  printf("data format             ... ");
  printf(dformat);
  printf("dimensions (x,y,z)      ... ");
  printf("(%d,%d,%d)\n",(int)((tlp/dx)+1),jmx+1,kmx+1);
  printf("orign (x,y,z)           ... ");
  printf("(%3.2f,%3.2f,%3.2f)\n",stp/twp,0.0,0.0);
  printf("spacing (x,y,z)         ... ");
  printf("(%3.2f,%3.2f,%3.2f)\n",dx/twp,dy/twp,dz/twp);
  printf("point_data              ... %d\n",pdata);
  printf("scalars                 ... ");
  printf(scalar);
  printf("lookup table            ... ");
  printf(lookup);
  printf("---------------------------------------------------------\n");
#endif
}

void out_data(void)
{
  int i,j,k,m=0;
  int cnt,gmx;
  double bb;
  
  /********** setup region            **********/
  cnt = (int)(glp/dx) - (int)(stp/dx);
  if(cnt*(jmx+1)*(kmx+1) == pdata)
    gmx = (int)(glp/dx);
  else
    gmx = (int)(glp/dx) + 1;
  
  /********** writing data            **********/
  for(k=0;k<=kmx;k++){
    for(j=0;j<=jmx;j++){
      for(i=(int)(stp/dx);i<gmx;i++)
	{
	  bb = abv(Fx[i][j][k],Fy[i][j][k],Fz[i][j][k]);
	  
	  if(i != gmx - 1)
	    fprintf(fp,"%f ",bb);
	  else
	    fprintf(fp,"%f", bb);
	  m++;
	}
      fprintf(fp,"\t");
    }
    fprintf(fp,"\n");
  }
#ifdef DEBUG
  printf("writing counter         ... %d\n",m);
#endif
  /********** check errors            **********/
  if(m != pdata)
    printf("***** no match point_data *****\n");  
}

double abv(double xx,double yy,double zz)
{
  return sqrt(xx*xx + yy*yy + zz*zz);
}

/************************** shell option command *****************************/
void getopt(int opt,char *buf[],int argc,char *argv[])
{
  int i = 0;
  
  do{
    if(i > FBUF)
      {
        printf("Argv chars are too big ...\n");
        exit(1);
      }
    
    buf[i] = (argv[opt]+i);
    i++;
  }while(*(argv[opt]+i) != '\0');
  
  *(buf[0]+i) = '\0';
}

int startup(int argc,char *argv[])
{
  char *buf[FBUF+1];
  int i = 0;
  
  if(argc < 2)
    printf("No parameter ...\n");
  else if(argc >= 3)
    printf("A lot of parameter ...\n");
  else
    {
      /* get and arrange arguments */
      getopt(1,buf,argc,argv);
      
      while(*(buf[0]+i) != '\0')
        {
          fnm[i] = *buf[i];
          i++;
        }
      bf = i;
      return optSuc;
    }
  return optErr;
}
/*****************************************************************************
*                            dat2vtk exchange program                        *
*                                  for lpis3d version                        *
*                                           2005.9.27                        *
*                              Copyright  Keita Ogawa                        *
*****************************************************************************/
